﻿
using Microsoft.Win32;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Security.Permissions;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

using System.Timers;

struct tagUsartFrameInfo
{
    public byte nCmd;
    public byte nCH;
    public byte nLight3Bit;
    public byte nLight2Bit;
    public byte nLight1Bit;
    public byte nXORHBit;
    public byte nXORLBit;
    public byte nCheckSum;
}; 

namespace PCSoftware_demo
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {

        public const int DBS_SUCCEED = 0;		// 操作成功
        public const int DBS_ERR_INVALIDHANDLE=	1001;	// 无效的操作句柄
        public const int DBS_ERR_UNKNOWN=1002;	// 未知错误
        public const int DBS_ERR_INITSERIALPORT_FAILED=1003;	// 初始化串口失败
        public const int DBS_ERR_RELEASESERIALPORT_FAILED=1004;	// 释放串口错误
        public const int DBS_ERR_SERIALPORT_UNOPENED=1005;	// 试图操作一个未打开的串口
        public const int DBS_ERR_TURNON_CH_FAILED=1006;	// 打开通道错误
        public const int DBS_ERR_TURNOFF_CH_FAILED=1007;	// 关闭通道错误
        public const int DBS_ERR_SET_INTENSITY_FAILED=1008;	// 设置亮度错误
        public const int DBS_ERR_READ_INTENSITY_FAILED=1009;	// 读取亮度错误
        public const int DBS_ERR_SET_WORKMODE_FAILED=1010;	// 设置工作模式错误
        public const int DBS_ERR_READ_WORKMODE_FAILED=1011;	// 读取工作模式错误
        public const int DBS_ERR_INVALIDCMD=1012;	// 无效的指令
        public const int DBS_ERR_VERIFY_FAILED=1013;	// 指令校验错误
        public const int DBS_ERR_COMM_TIMEOUT=1014;	// 通信超时错误
        public const int DBS_ERR_BACK_DATA=1015;	// 返回数据错误
        public const int DBS_ERR_COMM_TOO_QUICK=1016;	// 指令速度过快


        public const int  CMD_SET_INTENSITY	=3;
        public const int  CMD_GET_INTENSITY=4;
        public const int  CMD_TURNON_CHANNEL=1;
        public const int  CMD_TURNOFF_CHANNEL=2;
        public const int  CMD_SET_FLUSH_FIRE=7;	// 开启一次频闪
        public const int  CMD_SET_TRICK_MODE=8;
        public const int  CMD_SET_FLUSH_TIME=9;
        public const int  CMD_GET_SERIAL_ID	=6;		// 获取序列ID
        public const int  CMD_SET_SPECIFIED_DATA=0;		// 设置特殊数据
        public const int  CMD_SET_INT_PERIOD=3;	// 设置内触发周期
        public const int  CMD_SET_CAMERA_DELYA=17;		// 设置相机延时时间

        public const int   TRICK_MODE_LEVEL=1;		// 电平触发模式
        public const int   TRICK_MODE_NORMAL=0;		// 常亮模式
        public const int   TRICK_MODE_FLUSH_MS=2;		// 毫秒频闪模式
        public const int   TRICK_MODE_FLUSH_US=3;		// 微秒频闪模式

        tagUsartFrameInfo m_tUsartFrameInfo;

        Queue<byte> m_pRxBuf = new Queue<byte>();
        private bool recStaus = true;//接收状态字
        private bool ComPortIsOpen = false;//COM口开启状态字，在打开/关闭串口中使用，这里没有使用自带的ComPort.IsOpen，因为在串口突然丢失的时候，ComPort.IsOpen会自动false，逻辑混乱
        private bool Listening = false;//用于检测是否没有执行完invoke相关操作，仅在单线程收发使用，但是在公共代码区有相关设置，所以未用#define隔离
        private bool WaitClose = false;//invoke里判断是否正在关闭串口是否正在关闭串口，执行Application.DoEvents，并阻止再次invoke ,解决关闭串口时，程序假死，具体参见http://news.ccidnet.com/art/32859/20100524/2067861_4.html 仅在单线程收发使用，但是在公共代码区有相关设置，所以未用#define隔离
        System.Timers.Timer aTimer = new System.Timers.Timer();
        Socket tcpClient;
        IPAddress ipaddress = IPAddress.Parse("192.168.1.200");
        byte[] recBuffer = new byte[1024];//接收数据缓存  
        //EndPoint Endpoint = null;

        public MainWindow()
        {
            InitializeComponent();
        }

        public class customer//各下拉控件访问接口
        {

            public string com { get; set; }//可用串口
            public string com1 { get; set; }//可用串口
            public string BaudRate { get; set; }//波特率
            public string Parity { get; set; }//校验位
            public string ParityValue { get; set; }//校验位对应值
            public string Dbits { get; set; }//数据位
            public string Sbits { get; set; }//停止位


        }


        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
           
        }

        public void ReceiveMessage(IAsyncResult ar)
        {
            
            if (WaitClose) return;//如果正在关闭串口，则直接返回
            if (recStaus)//如果已经开启接收
            {
                try
                {
                    //Listening = true;////设置标记，说明我已经开始处理数据，一会儿要使用系统UI的。
     
                    string recData;//接收数据转码后缓存

                    if (tcpClient.Handle == null) return;

                    int receiveLength = tcpClient.EndReceive(ar);

                    if (receiveLength > 0)
                    {
                        for (int i = 0; i < receiveLength; i++)
                        {
                            m_pRxBuf.Enqueue(recBuffer[i]);
                        }

                        recData = System.Text.Encoding.Default.GetString(recBuffer, 0, receiveLength) + '\r' + '\n';//转码
                        this.TX_Message.Dispatcher.Invoke(//WPF为单线程，此接收中断线程不能对WPF进行操作，用如下方法才可操作
                        new Action(
                             delegate
                             {
                                 TX_Message.Text += recData;//加显到接收区

                             }
                        )
                        );

                    }

                    tcpClient.BeginReceive(recBuffer, 0, recBuffer.Length, SocketFlags.None, new AsyncCallback(ReceiveMessage), tcpClient);
                }
                catch
                {
                    MessageBox.Show("网络已断开");
                    //Listening = false;//UI使用结束，用于关闭串口时判断，避免自动发送时拔掉串口，陷入死循环
                }

            }
            else//暂停接收
            {
                
                //tcpClient.Shutdown(SocketShutdown.Both);
            }

        }  


        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            MessageBoxResult result = MessageBox.Show("确认是否要退出？", "退出", MessageBoxButton.YesNo);//显示确认窗口
            if (result == MessageBoxResult.No)
            {
                e.Cancel = true;//取消操作
            }
        }

        private void Btn_ComOpen_Click(object sender, RoutedEventArgs e)
        {

            if (TX_IPADDR.Text != "" || TX_IPPORT.Text != "")
            {
                ipaddress = IPAddress.Parse(TX_IPADDR.Text);
                tcpClient = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                #region 打开网口
                if (ComPortIsOpen == false)//ComPortIsOpen == false当前为关闭状态，按钮事件为打开
                {

                    try//尝试打开串口
                    {
                        tcpClient.Connect(ipaddress, Convert.ToInt32(TX_IPPORT.Text));
                       // MessageBox.Show("服务器连接中。。。");
                    }
                    catch//如果被其他占用，则无法打开
                    {
                        MessageBox.Show("服务器连接失败。。。");
                        return;//提示后直接返回
                    }

                    try
                    {
                        if (tcpClient.Connected == true)
                        {
                            MessageBox.Show("与服务器连接成功");
                            tcpClient.BeginReceive(recBuffer, 0, recBuffer.Length, SocketFlags.None, new AsyncCallback(ReceiveMessage), tcpClient);

                            //↓↓↓↓↓↓↓↓↓成功连接后的设置↓↓↓↓↓↓↓↓↓
                            Btn_ComOpen.Content = "断开";//按钮显示改为“关闭按钮”
                            WaitClose = false;//等待状态改为false  
                            ComPortIsOpen = true;//打开状态字改为true             
                            //↑↑↑↑↑↑↑↑↑成功连接后的设置↑↑↑↑↑↑↑↑↑
                        }
                        else
                        {
                            MessageBox.Show("与服务器连接失败");
                        }
                    }
                    catch
                    {
                        MessageBox.Show("检测连接状态出错");
                    }

                }
                #endregion
                #region 断开连接
                else//ComPortIsOpen == true,当前串口为打开状态，按钮事件为关闭串口
                {
                    try//尝试断开连接
                    {
                        WaitClose = true;//激活正在关闭状态字

                        while (Listening)//判断invoke是否结束
                        {
                            DispatcherHelper.DoEvents(); //循环时，仍进行等待事件中的进程，该方法为winform中的方法，WPF里面没有，这里在后面自己实现
                        }
                        tcpClient.Shutdown(SocketShutdown.Both);

                    }
                    catch
                    {
                        MessageBox.Show("已断开连接");
                        tcpClient.Close();
                        tcpClient.Dispose();
                        WaitClose = false;//关闭正在关闭状态字
                        SetAfterClose();//成功关闭后的设置
                        return;//提示后直接返回
                    }
                }
                #endregion
            }

           
        }

        private void SetAfterClose()//成功关闭串口或串口丢失后的设置
        {

            Btn_ComOpen.Content = "连接";//按钮显示为“打开串口”
            ComPortIsOpen = false;//状态设置为关闭状态
        }


        //模拟 Winfrom 中 Application.DoEvents() 详见 http://www.silverlightchina.net/html/study/WPF/2010/1216/4186.html?1292685167
        public static class DispatcherHelper
        {
            [SecurityPermissionAttribute(SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
            public static void DoEvents()
            {
                DispatcherFrame frame = new DispatcherFrame();
                Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.Background, new DispatcherOperationCallback(ExitFrames), frame);
                try { Dispatcher.PushFrame(frame); }
                catch (InvalidOperationException) { }
            }
            private static object ExitFrames(object frame)
            {
                ((DispatcherFrame)frame).Continue = false;
                return null;
            }
        }




        /* \函数内容 设置光源通道的亮度
        * \输入参数 nCH:通道号 取值范围[1-4]
			nIntensity:亮度 取值范围[0-255][10进制]
        * \返回值   是否设置成功 1.设置成功 -1.设置失败
        */
         int SetIntensity(int nCH, int nIntensity)
        {

	        int nRtn = 0;
	
	        // 先清空接收队列
	        m_pRxBuf.Clear();

	        // 发送数据
	        nRtn = SendCmd(CMD_SET_INTENSITY, nCH, nIntensity);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:488] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待返回
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
                TX_Message.Text += "[LED:497] WaiteCmdFB:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 返回成功
	        return DBS_SUCCEED;
        }

        
        /*!
        * \函数名称 SetFlushTimeMS
        * \函数内容 设置频闪时间:MS级
        * \输入参数 nTime:时间 取值范围:1-99
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        */
        int SetFlushTimeMS(int nCH, int nTime)
        {

	        int nRtn = 0;

	        // 先将模式设置成MS触发模式
	        nRtn = SendCmd(CMD_SET_TRICK_MODE, nCH, TRICK_MODE_FLUSH_MS);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:392] SendCmd:"+ nRtn.ToString()+'\r'+'\n';

		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:497] WaiteCmdFB:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 设置频闪时间
	        nRtn = SendCmd(CMD_SET_FLUSH_TIME, nCH, nTime);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:410] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
                TX_Message.Text += "[LED:418] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }


        /*!
        * \函数名称 SetFlushTimeUS
        * \函数内容 设置频闪时间:US级
        * \输入参数 nTime:时间 取值范围:10-990[个位数无效]
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        */
        int SetFlushTimeUS(int nCH, int nTime)
        {
            // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 设置频闪时间
	        int nRtn = SendCmd(CMD_SET_FLUSH_TIME, nCH, nTime);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:439] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
                TX_Message.Text += "[LED:447] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 SetMixFlushTimeUS
        * \函数内容 设置频闪时间:US级
        * \输入参数 nTime:时间 取值范围:1-999
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        */
        int SetMixFlushTimeUS(int nCH, int nTime)
        {
            // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 设置频闪时间
	        int nRtn = SendCmd2(CMD_SET_FLUSH_TIME, nCH, nTime);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:467] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:475] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 SetIntFlushPeriod
        * \函数内容 设置内触发周期:MS级
        * \输入参数 nTime:时间 取值范围:10-999
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        */
        int SetIntFlushPeriod(int nCH, int nTime)
        {
            // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 先将模式设置成US触发模式
	        int nRtn = SendCmd2(CMD_SET_INT_PERIOD, nCH, nTime);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:495] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:503] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 SetCameraDelay
        * \函数内容 设置相机触发延时时间:US级
        * \输入参数 nTime:时间 取值范围:5-99
        * \返回值   设置相机触发延时时间成功 1.设置成功 -1.设置失败
        */
        int SetCameraDelay(int nCH, int nTime)
        {
            // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 先将模式设置成US触发模式
	        int nRtn = SendCmd2(CMD_SET_CAMERA_DELYA, nCH, nTime);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:523] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:531] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 SetFlushFire
        * \函数内容 设置频闪开启
        * \输入参数 nCH: 通道号
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        * \作者 WS058
        * \日期 2019-11-19
        */
        int SetFlushFire(int nCH)
        {
            // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 开启触发
	        int nRtn = SendCmd(CMD_SET_FLUSH_FIRE, nCH, 0);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:553] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:561] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 SetNormalMode
        * \函数内容 设置成普通模式，即常亮模式
        * \输入参数 nTime:时间 取值范围:10-990[个位数无效]
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        */
        int SetNormalMode(int nCH)
        {

	        // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 发送指令
	        int nRtn = SendCmd(CMD_SET_TRICK_MODE, nCH, TRICK_MODE_NORMAL);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:584] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:592] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 SetTriggerMode
        * \函数内容 设置成触发模式，即常灭模式
        * \输入参数 nTime:时间 取值范围:10-990[个位数无效]
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        * \作者 WS058
        * \日期 2019-9-20
        */
        int SetTriggerMode(int nCH)
        {
            // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        int nRtn = SendCmd(CMD_SET_TRICK_MODE, nCH, TRICK_MODE_LEVEL);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:613] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待指令回复
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:621] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 ReadIntensity
        * \函数内容 获取亮度
        * \输入参数 nCH:通道号 取值范围:1-4
        nIntensity: 数据指针
        * \返回值   设置频闪时间成功 1.设置成功 -1.设置失败
        */
        int ReadIntensity(int nCH, ref int nIntensity)
        {

	        // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 发送读取数据指令
	        int nRtn = SendCmd(CMD_GET_INTENSITY, nCH, 0);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:645] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待数据
	        nRtn = WaiteCmdFB(CMD_SET_INTENSITY);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:653] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }
	
	        // 判断收到的数据的通道号是否正确
	        if (nCH != m_tUsartFrameInfo.nCH)
	        {
                TX_Message.Text += "[LED:439] CH Error:"+ nCH.ToString()+ '-'+ m_tUsartFrameInfo.nCH.ToString() +'\r'+'\n';
		        return DBS_ERR_UNKNOWN;
	        }

	        // 使用数据
	        nIntensity = (m_tUsartFrameInfo.nLight3Bit * 100) + (m_tUsartFrameInfo.nLight2Bit * 10) + m_tUsartFrameInfo.nLight1Bit;


	        // 返回数据
	        return DBS_SUCCEED;
        }


        /*!
        * \函数名称 TurnOnChannel
        * \函数内容 打开通道
        * \输入参数 nCH:通道号 取值范围:1-4
        * \返回值
        */
        int TurnOnChannel(int nCH)
        {

	        // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 发送设置指令
	        int nRtn = SendCmd(CMD_TURNON_CHANNEL, nCH, 0);
	        if (nRtn != DBS_SUCCEED)
	        {
		       TX_Message.Text += "[LED:689] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待返回指令
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:697] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 TurnOffChannel
        * \函数内容 打开通道
        * \输入参数 nCH:通道号 取值范围:1-4
        * \返回值
        */
        int TurnOffChannel(int nCH)
        {

	        // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        // 发送设置指令
	        int nRtn = SendCmd(CMD_TURNOFF_CHANNEL, nCH, 0);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:720] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待返回指令
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:728] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

        /*!
        * \函数名称 AdjustStartIntensity
        * \函数内容 启动调整起亮点
        * \输入参数 无
        * \返回值   启动调整起亮点是否成功 1.设置成功 -1.设置失败
        */
        int AdjustStartIntensity()
        {

	        // 清空接收缓冲区
	        m_pRxBuf.Clear();

	        int nSpecifyData1 = 7;		// 注册数据1
	        int nSpecifyData2 = 99;		// 注册数据2

	        int nRtn = SendCmd(CMD_SET_SPECIFIED_DATA, nSpecifyData1, nSpecifyData2);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:753] SendCmd:"+ nRtn.ToString()+'\r'+'\n';
		        return nRtn;
	        }

	        // 等待数据
	        nRtn = WaiteCmdFB(-1);
	        if (nRtn != DBS_SUCCEED)
	        {
		        TX_Message.Text += "[LED:761] WaiteCmdFB:" + nRtn.ToString() + '\r' + '\n';
		        return nRtn;
	        }

	        return DBS_SUCCEED;
        }

         public int SendCmd(int nCMD, int nCH, int nData)
         {

             byte[] nSendBuf = new byte[20];

             int nIndex = 0;
             int nSumCheck = 0;

             // 数据头
             nSendBuf[nIndex] = (byte)'$';
             nSumCheck = nSendBuf[nIndex];
             nIndex++;

             // 指令号
             nSendBuf[nIndex] = (byte)((nCMD % 256) + '0');
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;

             // 通道号或者信息1
             nSendBuf[nIndex] = (byte)((nCH % 256) + '0');
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;

             // 固定填充值
             nSendBuf[nIndex] = (byte)'0';
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;

             // 数据高位
             byte nDataH = (byte)((nData % 256) / 16);
             if (nDataH < 10) nDataH += (byte)'0';
             else nDataH += (byte)'A' - 10;
             nSendBuf[nIndex] = nDataH;
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;

             // 数据低位
             byte nDataL = (byte)((nData % 256) % 16);
             if (nDataL < 10) nDataL += (byte)'0';
             else nDataL += (byte)('A' - 10);
             nSendBuf[nIndex] = nDataL;
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;

             // 校验高位
             nDataH = (byte)(nSumCheck / 16);
             if (nDataH < 10) nDataH += (byte)'0';
             else nDataH += (byte)('A' - 10);
             nSendBuf[nIndex++] = nDataH;

             // 校验低位
             nDataL = (byte)(nSumCheck % 16);
             if (nDataL < 10) nDataL += (byte)'0';
             else nDataL += (byte)('A' - 10);
             nSendBuf[nIndex++] = nDataL;


             try//尝试发送数据
              {

                  tcpClient.Send(nSendBuf, nIndex, SocketFlags.None);
                  //ComPort.Write(nSendBuf, 0, nSendBuf.Length);

              }
              catch//如果无法发送，产生异常
              {
                   MessageBox.Show("无法发送数据，原因未知！");
                   return DBS_ERR_SERIALPORT_UNOPENED;
              }

             return DBS_SUCCEED;
         }


        int SendCmd2(int nCMD, int nCH, int nData)
        {

	        byte[] nSendBuf = new byte[20];

             int nIndex = 0;
             int nSumCheck = 0;

	        // 数据头
             nSendBuf[nIndex] = (byte)'$';
             nSumCheck = nSendBuf[nIndex];
             nIndex++;


            // 指令号
             nSendBuf[nIndex] = (byte)((nCMD % 256) + '0');
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;

             // 通道号或者信息1
             nSendBuf[nIndex] = (byte)((nCH % 256) + '0');
             nSumCheck ^= nSendBuf[nIndex];
             nIndex++;


	        // 数据百位
	        nSendBuf[nIndex] = (byte)(nData/100+'0');
	        nSumCheck ^= nSendBuf[nIndex];
	        nIndex++;

	        // 数据十位
	        nSendBuf[nIndex] = (byte)(nData / 10 % 10 + '0');
	        nSumCheck ^= nSendBuf[nIndex];
	        nIndex++;

	        // 数据个位
	        nSendBuf[nIndex] = (byte)(nData % 10 + '0');
	        nSumCheck ^= nSendBuf[nIndex];
	        nIndex++;

	        // 校验高位
	        byte nDataH = (byte)(nSumCheck / 16);
	        if (nDataH < 10) nDataH += (byte)'0';
	        else nDataH +=(byte)('A' - 10);
	        nSendBuf[nIndex++] = nDataH;

	        // 校验低位
	        byte nDataL = (byte)(nSumCheck % 16);
	        if (nDataL < 10) nDataL += (byte)('0');
	        else nDataL += (byte)('A') - 10;
	        nSendBuf[nIndex++] = nDataL;

            try//尝试发送数据
            {
                tcpClient.Send(nSendBuf, nIndex, SocketFlags.None);
                //ComPort.Write(nSendBuf, 0, nSendBuf.Length);

            }
            catch//如果无法发送，产生异常
            {
                MessageBox.Show("无法发送数据，原因未知！");
                return DBS_ERR_SERIALPORT_UNOPENED;
            }

	        return DBS_SUCCEED;
        }

        private int WaiteCmdFB(int nCMD)
        {

            int nRecvStep = 0;
	        byte nRecvData = 0;
            int TimeCnt = 0;
            byte[] nRecvBuf = new byte[10];	// 接收缓冲区
	        while (true)
	        {

                Thread.Sleep(10);

                TimeCnt++;

                if (TimeCnt > 100)
                {
                    return DBS_ERR_COMM_TIMEOUT;
                }

		        // 数据接收队列不为空
		        if (m_pRxBuf.Count > 0)
		        {
			        // 取数据
                    nRecvData = m_pRxBuf.Dequeue();
                    
			        // 处理数据
			        switch (nRecvStep)
			        {
			        case 0:		// 指令头
				        // 仅接收反馈信号，不接收数据
				        if (nCMD == -1)
				        {
					        // 校验成功
					        if (nRecvData == '$')
					        {
                                return DBS_SUCCEED;
					        }

					        // 校验失败
					        if (nRecvData == '&')
					        {
						        return DBS_ERR_VERIFY_FAILED;
					        }
				        }
				        else
				        {
					        // 收到指令头
					        if (nRecvData == '$')
					        {
						        nRecvBuf[0] = nRecvData;
						        nRecvStep = 1;
					        }
				        }
				        break;
			        case 1:		// 指令号
				        if (nRecvData != nCMD + 0x30)
				        {
					        // 返回的指令号错误
					        nRecvStep = 0;
				        }
				        else
				        {
					        // 指令号匹配，下一步
					        nRecvBuf[1] = nRecvData;

					        // 获取序列ID指令格式不同
					        if (nCMD != CMD_GET_SERIAL_ID)
					        {
						        nRecvStep = 2;
					        }
					        else
					        {
						        nRecvStep = 11;
						        m_tUsartFrameInfo.nCmd = (byte)nCMD;
					        }
				        }
				        break;
			        case 2:		// 通道号
				        if (nRecvData - 0x30 >= 10)
				        {
					        nRecvStep = 0;
				        }
				        else
				        {
					        nRecvBuf[2] = nRecvData;
					        m_tUsartFrameInfo.nCH = (byte)(nRecvData - 0x30);
					        nRecvStep = 3;
				        }
				        break;
			        case 3:		// 获取固定填充数据0
				        if (nRecvData >= '0' && nRecvData <= '9')
				        {
                            m_tUsartFrameInfo.nLight3Bit = (byte)(nRecvData - '0');
				        }
				        else
				        {
					        return DBS_ERR_BACK_DATA;
				        }

				        nRecvBuf[3] = nRecvData;
				        nRecvStep = 4;
				        break;
			        case 4:		// 接收数据高位
				        if (nRecvData >= '0' && nRecvData <= '9')
				        {
                            m_tUsartFrameInfo.nLight2Bit = (byte)(nRecvData - '0');
				        }
				        else
				        {
					        return DBS_ERR_BACK_DATA;
				        }

				        nRecvBuf[4] = nRecvData;
				        nRecvStep = 5;
				        break;
			        case 5:		// 接收数据低位
				        if (nRecvData >= '0' && nRecvData <= '9')
				        {
                            m_tUsartFrameInfo.nLight1Bit = (byte)(nRecvData - '0');
				        }
				        else
				        {
					        return DBS_ERR_BACK_DATA;
				        }

				        nRecvBuf[5] = nRecvData;
				        nRecvStep = 6;
				        break;
			        case 6:		// 接收校验高位
				        if (nRecvData >= '0' && nRecvData <= '9')
				        {
                            m_tUsartFrameInfo.nXORHBit = (byte)(nRecvData - '0');
				        }
				        else if (nRecvData >= 'A' && nRecvData <= 'F')
				        {
                            m_tUsartFrameInfo.nXORHBit = (byte)(nRecvData - 'A' + 10);
				        }
				        else
				        {
					        return DBS_ERR_BACK_DATA;
				        }
				        nRecvStep = 7;
				        break;
			        case 7:		// 接收校验低位
				        if (nRecvData >= '0' && nRecvData <= '9')
				        {
                            m_tUsartFrameInfo.nXORLBit = (byte)(nRecvData - '0');
				        }
				        else if (nRecvData >= 'A' && nRecvData <= 'F')
				        {
                            m_tUsartFrameInfo.nXORLBit = (byte)(nRecvData - 'A' + 10);
				        }
				        else
				        {
					        return DBS_ERR_BACK_DATA;
				        }
				

					    int nXORCheckCnt = nRecvBuf[0];
					    for (int i = 1; i < 6; i++)
					    {
						    nXORCheckCnt ^= nRecvBuf[i];
					    }

					    int nXORCheck = m_tUsartFrameInfo.nXORHBit * 16 + m_tUsartFrameInfo.nXORLBit;
					    if (nXORCheck != nXORCheckCnt)
					    {
						    return DBS_ERR_VERIFY_FAILED;
					    }

					    return DBS_SUCCEED;
			
				        //break;

			        default:
				        break;
			        }
		        }

	        }

        }

        private void SL_CH1_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(1, (int)SL_CH1.Value);
        }

        private void Btn_CH1_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(1);
        }

        private void Btn_CH1_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(1);
        }

        private void Btn_CH1_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(1, ref Intensity);
            TX_CH1.Text = Intensity.ToString();
        }

        private void Btn_CH1_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(1);
        }

        private void Btn_CH1_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if(ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(1);
                        break;
                    case 1:
                        SetNormalMode(1);
                        break;
                    case 2:
                        SetFlushTimeMS(1, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(1, ModeSet.FlusTime);
                        break;  
                    default:
                        break;
                }
                    
            }
        }

        private void SL_CH2_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(2, (int)SL_CH2.Value);
        }

        private void Btn_CH2_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(2);
        }

        private void Btn_CH2_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(2);
        }

        private void Btn_CH2_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(2, ref Intensity);
            TX_CH2.Text = Intensity.ToString();
        }

        private void Btn_CH2_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(2);
        }

        private void Btn_CH2_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(2);
                        break;
                    case 1:
                        SetNormalMode(2);
                        break;
                    case 2:
                        SetFlushTimeMS(2, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(2, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

        private void SL_CH3_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(3, (int)SL_CH3.Value);
        }

        private void Btn_CH3_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(3);
        }

        private void Btn_CH3_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(3);
        }

        private void Btn_CH3_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(3, ref Intensity);
            TX_CH3.Text = Intensity.ToString();
        }

        private void Btn_CH3_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(3);
        }

        private void Btn_CH3_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(3);
                        break;
                    case 1:
                        SetNormalMode(3);
                        break;
                    case 2:
                        SetFlushTimeMS(3, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(3, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

        private void SL_CH4_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(4, (int)SL_CH4.Value);
        }

        private void SL_CH5_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(5, (int)SL_CH5.Value);
        }

        private void SL_CH6_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(6, (int)SL_CH6.Value);
        }

        private void SL_CH7_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(7, (int)SL_CH7.Value);
        }

        private void SL_CH8_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SetIntensity(8, (int)SL_CH8.Value);
        }

        private void Btn_CH4_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(4);
        }

        private void Btn_CH5_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(5);
        }

        private void Btn_CH6_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(6);
        }

        private void Btn_CH7_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(7);
        }

        private void Btn_CH8_Open_Click(object sender, RoutedEventArgs e)
        {
            TurnOnChannel(8);
        }

        private void Btn_CH4_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(4);
        }

        private void Btn_CH5_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(5);
        }

        private void Btn_CH6_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(6);
        }

        private void Btn_CH7_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(7);
        }

        private void Btn_CH8_Close_Click(object sender, RoutedEventArgs e)
        {
            TurnOffChannel(8);
        }

        private void Btn_CH4_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(4, ref Intensity);
            TX_CH4.Text = Intensity.ToString();
        }

        private void Btn_CH5_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(5, ref Intensity);
            TX_CH5.Text = Intensity.ToString();
        }

        private void Btn_CH6_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(6, ref Intensity);
            TX_CH6.Text = Intensity.ToString();
        }

        private void Btn_CH7_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(7, ref Intensity);
            TX_CH7.Text = Intensity.ToString();
        }

        private void Btn_CH8_Read_Click(object sender, RoutedEventArgs e)
        {
            int Intensity = 0;
            ReadIntensity(8, ref Intensity);
            TX_CH8.Text = Intensity.ToString();
        }

        private void Btn_CH4_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(4);
        }

        private void Btn_CH5_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(5);
        }

        private void Btn_CH6_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(6);
        }

        private void Btn_CH7_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(7);
        }

        private void Btn_CH8_TRIG_Click(object sender, RoutedEventArgs e)
        {
            SetFlushFire(8);
        }

        private void Btn_CH4_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(4);
                        break;
                    case 1:
                        SetNormalMode(4);
                        break;
                    case 2:
                        SetFlushTimeMS(4, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(4, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

        private void Btn_CH5_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(5);
                        break;
                    case 1:
                        SetNormalMode(5);
                        break;
                    case 2:
                        SetFlushTimeMS(5, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(5, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

        private void Btn_CH6_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(6);
                        break;
                    case 1:
                        SetNormalMode(6);
                        break;
                    case 2:
                        SetFlushTimeMS(6, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(6, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

        private void Btn_CH7_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(7);
                        break;
                    case 1:
                        SetNormalMode(7);
                        break;
                    case 2:
                        SetFlushTimeMS(7, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(7, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

        private void Btn_CH8_MODE_Click(object sender, RoutedEventArgs e)
        {
            ModWindow ModeSet = new ModWindow();
            ModeSet.Owner = this;//赋予主窗口，子窗口打开后，再次点击主窗口，子窗口闪烁
            ModeSet.ShowDialog();

            if (ModeSet.mUpdata == true)
            {
                switch (ModeSet.mode)
                {
                    case 0:
                        SetTriggerMode(8);
                        break;
                    case 1:
                        SetNormalMode(8);
                        break;
                    case 2:
                        SetFlushTimeMS(8, ModeSet.FlusTime);
                        break;
                    case 3:
                        SetFlushTimeUS(8, ModeSet.FlusTime);
                        break;
                    default:
                        break;
                }

            }
        }

    }
}
